/// T-Rex : Copyright 2021 : Aldrin Mathew
///
/// AAF DEPENDABLE LICENSE - 1.0
///
/// This library is licensed under the AAF Dependable License 1.0.
/// You are allowed to use this library as a dependency in your
/// project(s) WITHOUT having to release/make available your own
/// project(s) under the same license.
///
/// Unless required by applicable law, this library is provided
/// "AS IS", WITHOUT ANY WARRANTIES OR CONDITIONS OF ANY KIND,
/// either express or implied. The author(s) of the project is
/// not liable for any harms, errors or troubles caused by using
/// this library, unless implied by law. By using this license,
/// you are acknowledging the complete terms and conditions of
/// licensing of this library as specified in AAF Dependable
/// License 1.0 available at this URL:
///
/// https://aldrinsartfactory.com/licenses/dependable/
///
/// You are NOT ALLOWED to sell, or distribute this
/// library under commercial or non-commercial purposes, other than
/// using it in your project. You are not allowed to revamp, build
/// upon, rebrand, refactor the contents of this library.
/// Your project shall contain "original work", ideally, the
/// definition of which is agreed upon by the community. Using
/// this library to achieve that is not a violation of this license.
/// You are NOT ALLOWED to use the name, branding and identity of
/// this library to identify or brand your own project. You ARE
/// however allowed to use the name and branding to pinpoint/show
/// the source of the contents/code/logic of your project. At no
/// point shall you use the identification of the Authors of this
/// library to associate them to your own projects, in a way that
/// is deceiving/misleading or gives out false information.
///

part of trex;

/// `Tx<T>` is a Trackable variable that can used to set callbacks which can be called before and after changes are made to the variable.
///
/// ```dart
/// T value
/// ```
/// Is the actual value of the variable. It is split into a Getter and a Setter.
///
/// ```dart
/// void Function(T)? beforeChange
/// ```
/// Is called just before a change of value happens to the variable. This can be null, in which case, there will be no callback before change in value.
///
/// ```dart
/// void Function(T)? afterChange
/// ```
/// Is called immediately after a change of value of the variable happens. This can be null, in which case, there will be no callback after change in value.
class Tx<T> {
  /// The `Tx<T>` constructor. `Tx<T>` is a Trackable variable that can used to set callbacks which can be called before and after changes are made to the variable. The variable is of generic type `T` which means it can hold any type of value.
  ///
  /// ```dart
  /// T value
  /// ```
  /// Should be used to initialise the variable.
  ///
  /// ```dart
  /// void Function(T)? beforeChange
  /// ```
  /// Is called just before a change of value happens to the variable. This can be null, in which case, there will be no callback before change in value.
  ///
  /// ```dart
  /// void Function(T)? afterChange
  /// ```
  /// Is called immediately after a change of value of the variable happens. This can be null, in which case, there will be no callback after change in value.
  Tx(
    T value, {
    this.beforeChange,
    this.afterChange,
  }) : this._value = value;

  /// The private variable holding the value
  T _value;

  /// The getter to get the value of the variable
  T get value => _value;

  /// The setter to set the value of the variable
  set value(T setValue) {
    /// Checks if [beforeChange] is null and calls with the current value
    if (this.beforeChange != null) {
      this.beforeChange!(this._value);
    }

    /// Sets the value of the variable
    this._value = setValue;

    /// Checks if [afterChange] is null and calls with the new value
    if (this.afterChange != null) {
      this.afterChange!(setValue);
    }

    for (int i = 0; i < this._callbacks.length; i++) {
      // this._uniqueCallbacks[i]._element.markNeedsBuild();
      this._callbacks[i]();
    }
  }

  /// This function will be called before the value of the variable is changed.
  void Function(T oldValue)? beforeChange;

  /// This function will be called after the value of the variable has been changed.
  void Function(T newValue)? afterChange;

  /// List of all callbacks registered with this variable.
  List<VoidCallback> _callbacks = [];

  /// This method registers a new UniqueWidgetCallback to this variable.
  void _addCallback(VoidCallback callback) {
    _callbacks.add(callback);
  }

  /// This method removes the UniqueWidgetCallback that matches the provided `id` and `widgetHash`.
  void _removeCallback({
    required VoidCallback widget,
  }) {
    int index = -1;
    for (int i = 0; i < this._callbacks.length; i++) {
      if (this._callbacks[i] == widget) {
        index = i;
      }
    }
    if (index != -1) {
      this._callbacks.remove(index);
    }
  }
}
